//
// advio.cpp : Defines the entry point for the console application.
// A C++ program demonstrating advanced conversions and
// formatting member functions.
// Copyright (c) Chris H. Pappas and William H. Murray, 2001
//

#include "stdafx.h"
#include <string.h>
#include <iostream>
#include <strstream>

using namespace std;

#define INULL_TERMINATOR 1

void row(void);

int main(int argc, char* argv[])
{
  char   c        =   'A',
         psz1[]   =   "In making a living today many no ", 
         psz2[]   =   "longer leave any room for life.";
  int    iln      =   0,
         ivalue   =   1234;
  double dPi      =   3.14159265;

  char psz_padstring5[5+INULL_TERMINATOR], 
  psz_padstring38[38+INULL_TERMINATOR];

  // conversions

  // print the c
  row( ); // [ 1]
  cout << c;

  // print the ASCII code for c
  row( ); // [ 2]
  cout << (int)c;

  // print character with ASCII 90
  row( ); // [ 3]
  cout << (char)90;

  // print ivalue as octal value
  row( ); // [ 4]
  cout << oct << ivalue;

  // print lower-case hexadecimal
  row( ); // [ 5]
  cout << hex << ivalue;

  // print upper-case hexadecimal
  row( ); // [ 6] cout.setf(ios::uppercase);
  cout << hex << ivalue; 
  cout.unsetf(ios::uppercase);    // turn uppercase off 
  cout << dec;                    // return to decimal base

  // conversions and format options

  // minimum width 1
  row( ); // [ 7]
  cout << c;

  // minimum width 5, right-justify
  row( ); // [ 8] 
  ostrstream(psz_padstring5,sizeof(psz_padstring5)) 
  << "    " << c << ends;
  cout << psz_padstring5;

  // minimum width 5, left-justify
  row( ); // [ 9] 
  ostrstream(psz_padstring5,sizeof(psz_padstring5)) 
  << c << "    " << ends;
  cout << psz_padstring5;

  // 33 automatically
  row( ); // [10]
  cout << psz1;

  // 31 automatically
  row( ); // [11]
  cout << psz2;

  // minimum 5 overriden, auto
  // notice that the width of 5 cannot be overridden!
  row( ); // [12]
  cout.write(psz1,5);

  // minimum width 38, right-justify
  // notice how the width of 38 ends with garbage data
  row( ); // [13]
  cout.write(psz1,38);

  // the following is the correct approach
  cout << "\n\nCorrected approach:\n"; 
  ostrstream(psz_padstring38,sizeof(psz_padstring38)) << "     " 
    << psz1 << ends;
  row( ); // [14]
  cout << psz_padstring38;

  // minimum width 38, left-justify
  ostrstream(psz_padstring38,sizeof(psz_padstring38)) 
    << psz2 << "       " << ends;
  row( ); // [15]
  cout << psz_padstring38;

  // default ivalue width
  row( ); // [16]
  cout << ivalue;

  // printf ivalue with + sign
  row( ); // [17]
  cout.setf(ios::showpos);     // don't want row number with + 
  cout << ivalue;
  cout.unsetf(ios::showpos);

  // minimum 3 overridden, auto
  row( ); // [18]
  cout.width(3); // don't want row number padded to width of 3 
  cout << ivalue;

  // minimum width 10, right-justify
  row( ); // [19]
  cout.width(10);    // only in effect for first value printed 
  cout << ivalue;

  // minimum width 10, left-justify
  row( ); // [20]
  cout.width(10);
  cout.setf(ios::left);
  cout << ivalue;
  cout.unsetf(ios::left);

  // right-justify with leading 0's
  row( ); // [21]
  cout.width(10);
  cout.fill('0');
  cout << ivalue;
  cout.fill(' ');

  // using default number of digits
  row( ); // [22]
  cout << dPi;

  // minimum width 20, right-justify 
  row( ); // [23]
  cout.width(20);
  cout << dPi;

  // right-justify with leading 0's
  row( ); // [24]
  cout.width(20);
  cout.fill('0');
  cout << dPi;
  cout.fill(' ');

  // minimum width 20, left-justify
  row( ); // [25]
  cout.width(20);
  cout.setf(ios::left);
  cout << dPi;

  // left-justify with trailing 0's
  row( ); // [26]
  cout.width(20);
  cout.fill('0');
  cout << dPi;
  cout.unsetf(ios::left);
  cout.fill(' ');

  // additional formatting precision

  // minimum width 19, print all 17
  row( ); // [27]
  cout << psz1;

  // prints first 2 chars
  row( ); // [28]
  cout.write(psz1,2);

  // prints 2 chars, right-justify
  row( ); // [29]
  cout << "                  "; cout.write(psz1,2);

  // prints 2 chars, left-justify
  row( ); // [30]
  cout.write(psz1,2);

  // using printf arguments
  row( ); // [31]
  cout << "              "; cout.write(psz1,6);

  // width 10, 8 to right of '.'
  row( ); // [32]
  cout.precision(9);
  cout << dPi;

  // width 20, 2 to right-justify
  row( ); // [33]
  cout.width(20);
  cout.precision(2);
  cout << dPi;

  // 4 decimal places, left-justify
  row( ); // [34]
  cout.precision(4);
  cout << dPi;

  // 4 decimal places, right-justify
  row( ); // [35]
  cout.width(20);
  cout << dPi;

  // width 20, scientific notation
  row( ); // [36] cout.setf(ios::scientific); cout.width(20);
  cout << dPi; cout.unsetf(ios::scientific);
  cout << endl;
  return(0);
}

void row (void)
{
  static int ln=0;
  cout << "\n[";
  cout.width(2);
  cout << ++ln << "] ";
}